#ifndef _HeirRoleVectorEvent_H_
#define _HeirRoleVectorEvent_H_

#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "GPDefs.hpp"
#include "Vector.hpp"
#include "Pair.hpp"
#include "NTLink.hpp"
#include "MoreAlgorithms.hpp"
#include "ImDom.hpp"
#include "HeirRoleVector.hpp"
#include <boost/serialization/serialization.hpp>

using namespace std;

/*Forward declaration*/
class NTLink;

/*Class Declaration*/

//! An HeirRoleVector model event. This model event keeps track of Heir Vectors conditioned on NTLinks and shorts (a valency).
class HeirRoleVectorEvent
  : public Pair< ImDom<NTLink, short>, HeirRoleVector> {
 public:
  typedef Pair< ImDom<NTLink,short>, HeirRoleVector> _super;
  typedef _super::FirstType FirstType;
  typedef _super::SecondType SecondType;

  /*
   * Constructors.
   */
  //! Empty constructor.
  HeirRoleVectorEvent();

  //! Default constructor.
  HeirRoleVectorEvent(const arity_t& dim);

  //! Useful constructor.
  HeirRoleVectorEvent(const FirstType l, const HeirRoleVector& v);

  virtual ~HeirRoleVectorEvent() {};

  arity_t arity() const { return first.parent().arity();}

  /*
   * Operators.
   */
  //! Equality test.
  bool operator == (const HeirRoleVectorEvent& other) const
  { assert(arity()); return Pair< FirstType, SecondType>::operator==(other);}

  //! Assignment.
  HeirRoleVectorEvent& operator = (const HeirRoleVectorEvent& other)
  { _super::operator=(other); return *this; }

  //! Ordering.
  bool operator > (const HeirRoleVectorEvent& other) const
  { return _super::operator > (other);}
  bool operator< (const HeirRoleVectorEvent& other) const
  { return _super::operator< (other);}

  /*
   * Member access.
   */
  //! Returns the link.
  NTLink& link() { return first.parent();}
  NTLink link() const { return first.parent();}

  //! Returns the valency value.
  short& valency() { return first.child();}
  short valency() const { return first.child();}

  HeirRoleVector& heirVector() {return second;}
  HeirRoleVector heirVector() const {return second;}

  FirstType& getParValPair() {return first;}
  FirstType getParValPair()const {return first;}

  void setLink(NTLink& lnt){first.parent() = lnt;}
  void setValency(short& val){first.child() = val;}
  void setParValPair(FirstType & id){first = id;}
  void setHeirRoleVector(HeirRoleVector & hv){second = hv;}

  bool headSplit()const {return second.headSplit();};

  /*
   * I/O
   */

  //! Output.
  friend ostream& operator<<(ostream& out, const HeirRoleVectorEvent& valE)
  {out<<valE.first<<PAIRDELIM<<valE.second; return out;}

  ostream& put(ostream& out) const{
    out<<first<<PAIRDELIM<<second;
    return out;
  }

  //! Reads line by line.
  friend istream& operator >> (istream& in, HeirRoleVectorEvent& hve) {

    in >> hve.first;
    in >> ws;
    in >> hve.second;
    return in;
  }

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object< _super >(*this);
  }

  //! For hashing.
  virtual size_t hash () const
  { return _super::hash(); }

};

/////////////////////////////////////////////////////////////////////////

namespace HashNS {
  //! Template specialization of STL's hash object (a hash function) for
  //! ValencyEvents.
  template<> struct hash<HeirRoleVectorEvent> {
    size_t operator() (const HeirRoleVectorEvent& vE) const{ return vE.hash();}
  };
}

#endif
